"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var typescript_1 = __importDefault(require("typescript"));
var const_1 = require("./const");
var type_1 = require("./type");
var formProperFilePath = function (_a) {
    var delimiter = _a.delimiter, filePath = _a.filePath;
    return filePath.split(delimiter).filter(Boolean).join(delimiter);
};
var checkJavaScriptFileExistByAppend = function (_a) {
    var filePath = _a.filePath;
    var _b = (0, const_1.extensionsUtil)().extensions, js = _b.js, mjs = _b.mjs;
    var jsFile = filePath.concat(js);
    if (fs_1.default.existsSync(jsFile)) {
        return js;
    }
    var mjsFile = filePath.concat(mjs);
    if (fs_1.default.existsSync(mjsFile)) {
        return mjs;
    }
    return false;
};
var isDirectory = function (filePath) {
    try {
        return fs_1.default.lstatSync(filePath).isDirectory();
    }
    catch (error) {
        return false;
    }
};
var addJSExtension = function (_a) {
    var filePath = _a.filePath, delimiter = _a.delimiter, importPath = _a.importPath;
    if ((0, const_1.extensionsUtil)().matchAnyJs(filePath)) {
        return {
            procedure: 'skip',
        };
    }
    if (!isDirectory(filePath)) {
        var extension_1 = checkJavaScriptFileExistByAppend({
            filePath: filePath,
        });
        if (!extension_1) {
            return {
                procedure: 'skip',
            };
        }
        return {
            procedure: 'proceed',
            importPath: formProperFilePath({
                delimiter: delimiter,
                filePath: "".concat(importPath).concat(extension_1),
            }),
            filePathImported: formProperFilePath({
                delimiter: delimiter,
                filePath: "".concat(filePath).concat(extension_1),
            }),
        };
    }
    var extension = checkJavaScriptFileExistByAppend({
        filePath: "".concat(filePath, "/index"),
    });
    if (!extension) {
        return {
            procedure: 'skip',
        };
    }
    return {
        procedure: 'proceed',
        importPath: formProperFilePath({
            delimiter: delimiter,
            filePath: "".concat(importPath, "/index").concat(extension),
        }),
        filePathImported: formProperFilePath({
            delimiter: delimiter,
            filePath: "".concat(filePath, "/index").concat(extension),
        }),
    };
};
var traverseAndUpdateFileWithJSExtension = function (files) { return function (sourceFile) {
    var charactersDelimiter = '';
    var characters = sourceFile.getText().split(charactersDelimiter);
    var replaceNodes = sourceFile.statements.flatMap(function (statement) {
        switch (statement.kind) {
            case typescript_1.default.SyntaxKind.ImportDeclaration:
            case typescript_1.default.SyntaxKind.ExportDeclaration: {
                var imExDeclaration_1 = statement;
                if (!imExDeclaration_1.moduleSpecifier) {
                    return [];
                }
                var moduleSpecifier = (0, type_1.asString)({
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-ignore
                    value: imExDeclaration_1.moduleSpecifier.text,
                    error: function () {
                        return new Error('Module specifier of node should have text');
                    },
                });
                var delimiter = '/';
                var fileName = formProperFilePath({
                    delimiter: delimiter,
                    filePath: !moduleSpecifier.endsWith(delimiter)
                        ? moduleSpecifier
                        : moduleSpecifier.slice(0, -1),
                })
                    .split(delimiter)
                    .pop();
                if (!fileName) {
                    throw new Error("Impossible for file name to be non-existent for ".concat(moduleSpecifier));
                }
                if (moduleSpecifier.startsWith('.')) {
                    var result = addJSExtension({
                        delimiter: delimiter,
                        importPath: moduleSpecifier,
                        filePath: path_1.default.posix.join(sourceFile.fileName, '..', moduleSpecifier),
                    });
                    switch (result.procedure) {
                        case 'proceed': {
                            // if file name not included in list of js file read
                            var filePathImported_1 = result.filePathImported, importPath = result.importPath;
                            if (files.find(function (file) {
                                return file.endsWith(filePathImported_1);
                            })) {
                                var before = characters
                                    .filter(function (_, index) {
                                    return index > imExDeclaration_1.pos &&
                                        index < imExDeclaration_1.end;
                                })
                                    .join(charactersDelimiter);
                                return [
                                    {
                                        before: before,
                                        after: before.replace(moduleSpecifier, importPath),
                                    },
                                ];
                            }
                        }
                    }
                }
            }
        }
        return [];
    });
    return !replaceNodes.length
        ? []
        : [
            {
                file: sourceFile.fileName,
                code: replaceNodes.reduce(function (prev, _a) {
                    var before = _a.before, after = _a.after;
                    return prev.replace(before, after);
                }, characters.join(charactersDelimiter)),
            },
        ];
}; };
exports.default = traverseAndUpdateFileWithJSExtension;
//# sourceMappingURL=traverse-and-update.js.map